@@ -1,16 +1,16 @@
 # Test parse headers
 
 # import functions
-parse_headers = RestRserve:::parse_headers
+cpp_parse_headers = RestRserve:::cpp_parse_headers
 
 # Empty input
-expect_error(parse_headers(NA))
-expect_error(parse_headers(NULL))
-expect_true(inherits(parse_headers(NA_character_), "list"))
-expect_equal(length(parse_headers(NA_character_)), 0L)
-expect_true(inherits(parse_headers(""), "list"))
-expect_equal(length(parse_headers("")), 0L)
-expect_error(parse_headers("param\u100: value"), "header contains invalid character.")
+expect_error(cpp_parse_headers(NA))
+expect_error(cpp_parse_headers(NULL))
+expect_true(inherits(cpp_parse_headers(NA_character_), "list"))
+expect_equal(length(cpp_parse_headers(NA_character_)), 0L)
+expect_true(inherits(cpp_parse_headers(""), "list"))
+expect_equal(length(cpp_parse_headers("")), 0L)
+expect_error(cpp_parse_headers("param\u100: value"), "header contains invalid character.")
 
 # Test fields
 # nolint start
@@ -28,7 +28,7 @@ Cache-Control: max-age=0\r\n
 Custom_name: custom value
 TE: Trailers\r\n\r\n"
 
-parsed = parse_headers(h)
+parsed = cpp_parse_headers(h)
 expect_true(inherits(parsed, "list"))
 expect_equal(length(parsed), 13L)
 expect_equal(parsed[["connection"]], "keep-alive")
